-- MPMoney
-- @author  RedeX01
-- Version 1.0
-- @date 08.10.2022

MPMoney = {};
MPMoney.modDir = g_currentModDirectory;

-- character table string
local b='ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/';

-- encoding
function enc(data)
    return ((data:gsub('.', function(x)
        local r,b='',x:byte()
        for i=8,1,-1 do r=r..(b%2^i-b%2^(i-1)>0 and '1' or '0') end
        return r;
    end)..'0000'):gsub('%d%d%d?%d?%d?%d?', function(x)
        if (#x < 6) then return '' end
        local c=0
        for i=1,6 do c=c+(x:sub(i,i)=='1' and 2^(6-i) or 0) end
        return b:sub(c+1,c+1)
    end)..({ '', '==', '=' })[#data%3+1])
end;


-- decoding
function dec(data)
    data = string.gsub(data, '[^'..b..'=]', '')
    return (data:gsub('.', function(x)
        if (x == '=') then return '' end
        local r,f='',(b:find(x)-1)
        for i=6,1,-1 do r=r..(f%2^i-f%2^(i-1)>0 and '1' or '0') end
        return r;
    end):gsub('%d%d%d?%d?%d?%d?%d?%d?', function(x)
        if (#x ~= 8) then return '' end
        local c=0
        for i=1,8 do c=c+(x:sub(i,i)=='1' and 2^(8-i) or 0) end
        return string.char(c)
    end))
end


function money_format(amount)
    local formatted = amount;
    while true do
        formatted, k = string.gsub(formatted, "^(-?%d+)(%d%d%d)", '%1.%2')
        if (k==0) then
            break
        end
    end
    return formatted;
end;


function money_minimise_format(amount)
    local formatted = "";
    amount = math.floor(amount);
    if (amount >= 1000000000) then
        local nbMilliard = (amount-amount%1000000000)/1000000000;
        amount = amount - (nbMilliard * 1000000000);
        formatted = formatted..tostring(math.floor(nbMilliard)).."G.";
    end;
    if (amount >= 1000000) then
        local nbMillion = (amount-amount%1000000)/1000000;
        amount = amount - (nbMillion * 1000000);
        formatted = formatted..tostring(math.floor(nbMillion)).."M.";
    end;
    local nbMillier = (amount-amount%1000)/1000;
    amount = amount - (nbMillier * 1000);
    formatted = formatted..tostring(math.floor(nbMillier)).."K";

    return formatted;
end;


function ranks_string_limit(username, money)
    local charLimit = 22;
    local strLimited = "";
    local strMoney = "("..money_minimise_format(money)..")";
    local strCutChar = "";
    if (string.len(strMoney)+string.len(username) > charLimit) then
        strCutChar = "...";
    end;
    strLimited = string.sub(username,0,charLimit-string.len(strMoney)-3)..strCutChar.." "..strMoney;
    return strLimited;
end;


function cleanup_string(str)
    str = str:gsub( "%W", "_" );
    str = str:gsub("%d", "_");
	str = str:gsub(",", "");
	str = str:gsub("|", "");
	str = str:gsub("#", "");
    return str;
end;


function spairs(t, order)
    -- collect the keys
    local keys = {}
    for k in pairs(t) do keys[#keys+1] = k end

    -- if order function given, sort by it by passing the table and keys a, b,
    -- otherwise just sort the keys
    if order then
        table.sort(keys, function(a,b) return order(t, a, b) end)
    else
        table.sort(keys)
    end

    -- return the iterator function
    local i = 0
    return function()
        i = i + 1
        if keys[i] then
            return keys[i], t[keys[i]]
        end
    end
end


function MPMoney:fLog(str)

    print(str);

end;


function MPMoney:loadMap(name)
    self.HUDActive = false;
    self.increaseAmount = 5000;
    self.username = "";
    self.bankMoney = 0;
    self.masterUsername = "";
	self.statsOffset = 0;
    self.addToServer = 0;
    self.addToBank = 0;
    self.addToOther = 0;
    self.selectedOtherIndex = 0;
    self.selectedOtherNickname = "";
    self.ranks = "";
	self.history = "";
	self.inputSelected = 0;
	self.showhistory = false;
    self.HUD = {}
    self.HUD.background_bank = createImageOverlay(MPMoney.modDir.."img/"..g_i18n:getText("IMG_BACKGROUND_BANK")..".dds");
	self.HUD.background_history = createImageOverlay(MPMoney.modDir.."img/"..g_i18n:getText("IMG_BACKGROUND_HISTORY")..".dds");
	self.filedir = getUserProfileAppPath();
    self.dbfilename = self.filedir.."mpmoney_db.xml";
    self.dbFileFound = false;
	getFiles(self.filedir, "getFilesCallback", self);
	self.userChecked = false;
end;


function MPMoney:checkDataFile()
	if not self.dbFileFound then
		self:createDataFile();
		self:fLog("## MPMoney ## Create new database file");
    end;
end;


function MPMoney:getFilesCallback(filename)
    if filename == "mpmoney_db.xml" then
        self.dbFileFound = true;
		self:fLog("## MPMoney ## Database file found");
    end;
end;


function MPMoney:createDataFile()
    local xml = createXMLFile("mpmoney_db", self.dbfilename, "mpmoney_db");
    saveXMLFile(xml);
    delete(xml);
    self.dbFileFound = true;
end;


function MPMoney:update(dt)

	if (g_gui.currentGui == nil and self.username == "") then
		-- Get username
		for a=1, table.name(g_currentMission.users) do
			local user = g_currentMission.users[a];
			if user.userId == g_currentMission.playerUserId then
				self.username = user.nickname;
				self.username = cleanup_string(self.username);
				break;
			end;
		end;
		-- Check if master
        if g_currentMission:getIsServer() then
            self.masterUsername = self.username;
        end;
		if (not g_currentMission:getIsServer()) then
			-- Send user to server
			self:SendToServer(self.username, "checkMe","");
			self.userChecked = true;
		end;
	end;

	g_currentMission:addHelpButtonText(g_i18n:getText("input_OPEN_BANK"), InputBinding.OPEN_BANK, nil, GS_PRIO_HIGH);

    if self.HUDActive then
        self.addToBank = math.max(0, math.min(self.addToBank, math.floor(g_currentMission.missionStats.money)));
        self.addToServer = math.max(0, math.min(self.addToServer, self.bankMoney));
		self.addToOther = math.max(0, math.min(self.addToOther, self.bankMoney));
        if InputBinding.(InputBinding.CLOSE_BANK) then
            self:leaveMenu();
        end;
		if not (g_gui.currentGui == self) then
			self:leaveMenu();
		end;
    else
        if InputBinding.(InputBinding.OPEN_BANK) then
            self:enterMenu();
        end;
    end;

end;


function MPMoney:enterMenu()

    if (self.HUDActive == false) then 
		if (g_currentMission:getIsServer()) then
			self:SendToServer(self.username, "checkMe","");
		end;
        self:SendToServer(self.username, "needRefreshBank", "");
        self:SendToServer(self.username, "needRefreshRanks", "");
		self:SendToServer(self.username, "needRefreshHistory", "");
        self.HUDActive = true;
        InputBinding.setShowMouseCursor(true);
        g_gui.currentGui = self;
        self.dbFileFound = true;
		self.inputSelected = 0;
    end;

end;


function MPMoney:leaveMenu()
    if (self.HUDActive) then
        self.HUDActive = false;
        InputBinding.setShowMouseCursor(false);
        g_gui.currentGui = nil;
    end;
end;


function MPMoney:draw()

   if self.HUDActive then
   
		if (self.showhistory) then
		
			renderOverlay(self.HUD.background_history, 0, 0, 1, 1);
			
			-- local fontSize = 0.070;
			-- setTextColor(1, 1, 1, 1);
			-- renderText(0.2216, 0.7703, fontSize, g_i18n:getText("HISTORY_TITLE"));
			
			setTextColor(0, 1, 1, 1);	
			local fontSize = 0.033;
			local offsetY = 0;
			local nbShowed = 0;
			local nbElement = 0;

			for playerInfos in string.gmatch(self.history, "([^#]+)") do
				if (nbElement > self.statsOffset) then
					if (nbShowed < 13) then
						local timestamp, datetime, transfertType, transfertSource, transfertTarget, transfertValue = playerInfos:match("([^,]+)|([^,]+)|([^,]+)|([^,]+)|([^,]+)|([^,]+)");
						local infos = "";
						
						if (transfertSource == self.username) then
							if (transfertSource == transfertTarget) then
								if (transfertType == "toBank") then
									infos = infos..datetime.." "..g_i18n:getText("HISTORY_TRANSFERT_OF").." "..money_format(math.floor(transfertValue)).." "..g_i18n:getText("HISTORY_TO_YOUR_BANK");
								end;
								if (transfertType == "toServer") then
									infos = infos..datetime.." "..g_i18n:getText("HISTORY_TRANSFERT_OF").." "..money_format(math.floor(transfertValue)).." "..g_i18n:getText("HISTORY_TO_YOUR_WALLET");
								end;
							else
								infos = infos..datetime.." "..g_i18n:getText("HISTORY_YOU_HAVE_SEND").." "..money_format(math.floor(transfertValue)).." "..g_i18n:getText("HISTORY_TO_BANK_OF").." "..transfertTarget;
							end;
						else
							infos = infos..datetime.." "..transfertSource.." "..g_i18n:getText("HISTORY_PLAYER_SEND_TO_YOU").." "..money_format(math.floor(transfertValue));
						end;
						renderText(0.1345, 0.6950-offsetY, fontSize, infos);
						offsetY = offsetY + 0.04534
					else
						break;
					end;
					nbShowed = nbShowed + 1;
				end;
				nbElement = nbElement + 1;
			end;
		
		
		else

			renderOverlay(self.HUD.background_bank, 0, 0, 1, 1);
			
			-- local fontSize = 0.070;
			-- setTextColor(1, 1, 1, 1);
			-- renderText(0.1396, 0.7654, fontSize, g_i18n:getText("FORM_TITLE"));
			
			-- local fontSize = 0.035;
			-- renderText(0.1503, 0.6715, fontSize, g_i18n:getText("TRANSFERT_TO_BANK"));
			-- renderText(0.1503, 0.5210, fontSize, g_i18n:getText("TRANSFERT_TO_WALLET"));
			-- renderText(0.1503, 0.3695, fontSize, g_i18n:getText("TRANSFERT_TO_ANOTHER"));
			-- renderText(0.6484, 0.8054, fontSize, g_i18n:getText("MONEY_IN_WALLET"));
			-- renderText(0.6484, 0.6891, fontSize, g_i18n:getText("MONEY_IN_BANK"));
			-- renderText(0.6484, 0.5133, fontSize, g_i18n:getText("RANKING_TITLE"));

			local fontSize = 0.036;
			setTextColor(1, 0.70, 0, 1);
			renderText(0.6572, 0.7437, fontSize, tostring(money_format(math.floor(g_currentMission.missionStats.money))));
			renderText(0.6572, 0.6301, fontSize, tostring(money_format(math.floor(self.bankMoney))));
			renderText(0.1765, 0.6033, fontSize, tostring(money_format(math.floor(self.addToBank))));
			renderText(0.1765, 0.4541, fontSize, tostring(money_format(math.floor(self.addToServer))));
			renderText(0.1765, 0.2094, fontSize, tostring(money_format(math.floor(self.addToOther))));

			setTextColor(0, 1, 1, 1);
			renderText(0.1765, 0.2901, fontSize, self.selectedOtherNickname);

			local offsetY = 0;
			local nbPlayer = 0;
			for playerInfos in string.gmatch(self.ranks, "([^#]+)") do
				if (nbPlayer < 7) then
					local username, money = string.match(playerInfos, "([^,]+)|([^,]+)");
					renderText(0.6572, 0.4440-offsetY, fontSize, ranks_string_limit(username, math.floor(money)));
					offsetY = offsetY + 0.04634
				else
					break;
				end;
				nbPlayer = nbPlayer + 1;
			end;

			setTextColor(1, 0.52, 1, 1);

		end;
			
    end;

end;


function MPMoney:mouseEvent(posX, posY, isDown, isUp, button)

    if self.HUDActive then
        if isUp then
		
			if (self.showhistory) then
				
				if posX > 0.8232 and posX < 0.8701 and posY > 0.7527 and posY < 0.8465 then
					self.showhistory = false;
				end;
				
				if posX > 0.8232 and posX < 0.8701 and posY > 0.6402 and posY < 0.7341 then
					if (self.statsOffset > 0) then
						self.statsOffset = self.statsOffset - 1;
					end;
				end;
				
				if posX > 0.8232 and posX < 0.8701 and posY > 0.1446 and posY < 0.2385 then
					self.statsOffset = self.statsOffset + 1;
				end;
			
			else
				
				if posX > 0.8398 and posX < 0.8686 and posY > 0.6119 and posY < 0.6725 then
					self.showhistory = true;
					self:SendToServer(self.username, "needRefreshHistory", "");
				end;
			
				if posX > 0.97 and posX < 1 and posY > 0.95 and posY < 1 then
					self:leaveMenu();
				end;	
				
				if posX > 0.1665 and posX < 0.3803 and posY > 0.5894 and posY < 0.6451 then
					self.inputSelected = 1;
				end;
				if posX > 0.1665 and posX < 0.3803 and posY > 0.4369 and posY < 0.4956 then
					self.inputSelected = 2;
				end;
				if posX > 0.1665 and posX < 0.3803 and posY > 0.1964 and posY < 0.2512 then
					self.inputSelected = 3;
				end;

				if posX > 0.4614 and posX < 0.5145 and posY > 0.5894 and posY < 0.6487 then
					self.addToBank = math.max(self.addToBank - self.increaseAmount, 0);
				end;
				if posX > 0.40 and posX < 0.4546 and posY > 0.5876 and posY < 0.6478 then
					self.addToBank = math.min(self.addToBank + self.increaseAmount, math.floor(g_currentMission.missionStats.money));
				end;
				if posX > 0.5218 and posX < 0.5760 and posY > 0.5894 and posY < 0.6497 then
					self:SendToServer(self.username, "sendMoney","toBank|"..self.username.."|"..self.addToBank);
					self.addToBank = 0;
				end;

				-- add to server
				if posX > 0.4609 and posX < 0.5151 and posY > 0.4402 and posY < 0.4977 then
					self.addToServer = math.max(self.addToServer - self.increaseAmount, 0);
				end;
				if posX > 0.3989 and posX < 0.4531 and posY > 0.4393 and posY < 0.4977 then
					self.addToServer = math.min(self.addToServer + self.increaseAmount, self.bankMoney);
				end;
				if posX > 0.5223 and posX < 0.5760 and posY > 0.4393 and posY < 0.4977 then
					self:SendToServer(self.username, "sendMoney","toServer|"..self.username.."|"..self.addToServer);
					self.addToServer = 0;
				end;



				-- next user
				if posX > 0.3963 and posX < 0.4510 and posY > 0.2789 and posY < 0.3383 then
					local nbOther = self:getCountNbOther();
					self.selectedOtherIndex = math.min(self.selectedOtherIndex + 1, nbOther);
					self.selectedOtherNickname = self:getSelectOtherIndexNickname(self.selectedOtherIndex);
				end;

				-- before user
				if posX > 0.4598 and posX < 0.5140 and posY > 0.2780 and posY < 0.3383 then
					self.selectedOtherIndex = math.max(self.selectedOtherIndex - 1, 1);
					self.selectedOtherNickname = self:getSelectOtherIndexNickname(self.selectedOtherIndex);
				end;



				-- more
				if posX > 0.3963 and posX < 0.4510 and posY > 0.1955 and posY < 0.2539 then
					self.addToOther = math.min(self.addToOther + self.increaseAmount, self.bankMoney);
				end;

				-- less
				if posX > 0.4588 and posX < 0.5125 and posY > 0.1955 and posY < 0.2539 then
					self.addToOther = math.max(self.addToOther - self.increaseAmount, 0);
				end;

				-- send
				if posX > 0.5192 and posX < 0.5739 and posY > 0.1964 and posY < 0.2548 then
					self:SendToServer(self.username, "sendMoney","toBank|"..self.selectedOtherNickname.."|"..self.addToOther);
					self.addToOther = 0;
				end;

			end;

        end;
    end;
end;


function MPMoney:getSelectOtherIndexNickname(index)
    local indexOther = 0;
    for playerInfos in string.gmatch(self.ranks, "([^#]+)") do
        local username, money = string.match(playerInfos, "([^,]+)|([^,]+)");
        if username ~= self.username then
            indexOther = indexOther + 1;
            if index == indexOther then
                return username;
            end;
        end;
    end;
    return "";
end;


function MPMoney:getCountNbOther()
    local nbOther = 0;
    for playerInfos in string.gmatch(self.ranks, "([^#]+)") do
        local username, money = string.match(playerInfos, "([^,]+)|([^,]+)");
        if username ~= self.username then
            nbOther = nbOther + 1;
        end;
    end;
    return nbOther;
end;


function MPMoney:getBankMoney(xml, username)
    --local clean_username = cleanup_string(tostring(username));
    local bankMoney = Utils.getNoNil(getXMLString(xml, "mpmoney_db."..tostring(username)), "MA==");
    bankMoney = dec(bankMoney);
    bankMoney = math.floor(tonumber(bankMoney));
    return bankMoney;
end;


function MPMoney:setBankMoney(xml, username, money)
	if (money < 0) then
		money = 0;
	end;
    --local clean_username = cleanup_string(tostring(username));
    local enc_money = tostring(money);
    enc_money = enc(enc_money);
    setXMLString(xml, "mpmoney_db."..tostring(username), enc_money);
    saveXMLFile(xml);
    if (self:isMaster(username)) then
        self.bankMoney = money;
    end;
end;


function MPMoney:getWalletMoney(username)
	local WalletMoney = 0;
	if self:isMaster(username) then
		WalletMoney = math.floor(g_currentMission.missionStats.money);
	else
		for a=1, table.getn(g_currentMission.users) do
			if cleanup_string(g_currentMission.users[a].nickname) == username then
				WalletMoney = math.floor(g_currentMission.users[a].money);
				break;
			end;
		end;
	end;
    return WalletMoney;
end;


function MPMoney:setWalletMoney(username, money)
	if (money < 0) then
		money = 0;
	end;
	if self:isMaster(username) then
		g_currentMission.missionStats.money = money;
	else
		for a=1, table.getn(g_currentMission.users) do
			if cleanup_string(g_currentMission.users[a].nickname) == username then
				g_currentMission.users[a].money = money;
				break;
			end;
		end;
	end;
end;


function MPMoney:isMaster(username)
	if not (username == "") then
		if g_currentMission:getIsServer() then
			self.masterUsername = self.username;
		end;
		if (username == self.masterUsername) then
			return true;
		end;
		for a=1, table.getn(g_currentMission.users) do
			if g_currentMission.users[a].nickname == username then
				if g_currentMission.users[a].isMasterUser then
					self.masterUsername = username;
					return true;
				end;
			end;
		end;
	end;
    return false;	
end;


function MPMoney:addInHistory(xml, transfertType, transfertSource, transfertTarget, transfertValue)

	
	local infos = getDate("%y%m%d%H%M%S").."|"..getDate("%d-%m-%y %H:%M").."|"..transfertType.."|"..transfertSource.."|"..transfertTarget.."|"..transfertValue.."#";
	
	local clean_username = cleanup_string(tostring(transfertSource));
	local transferInfos = Utils.getNoNil(getXMLString(xml, "mpmoney_db."..tostring(clean_username).."_history"), "");
	transferInfos = dec(transferInfos);
	transferInfos = infos..transferInfos;
	transferInfos = enc(transferInfos);
	setXMLString(xml, "mpmoney_db."..tostring(clean_username).."_history", transferInfos);
	saveXMLFile(xml);
	self:SendToServer(transfertSource, "needRefreshHistory", "");
	
	if not (transfertSource == transfertTarget) then
		clean_username = cleanup_string(tostring(transfertTarget));
		transferInfos = Utils.getNoNil(getXMLString(xml, "mpmoney_db."..tostring(clean_username).."_history"), "");
		transferInfos = dec(transferInfos);
		transferInfos = infos..transferInfos;
		transferInfos = enc(transferInfos);
		setXMLString(xml, "mpmoney_db."..tostring(clean_username).."_history", transferInfos);
		saveXMLFile(xml);
		self:SendToServer(transfertTarget, "needRefreshHistory", "");
	end;
	
end;


function MPMoney:SendToClient(actionTarget, actionType, actionData)

	if g_currentMission:getIsServer() then
		if (actionTarget == self.username or actionTarget == "%all") then
			self:ClientController(actionTarget, actionType, actionData);
		end;
		if not (type(actionTarget)=="string")then
			actionTarget = tostring(actionTarget);
		end;
		if not (type(actionType)=="string")then
			actionType = tostring(actionType);
		end;
		if not (type(actionData)=="string")then
			actionData = tostring(actionData);
		end;
		ClientControllerEvent.sendEvent(actionTarget, actionType, actionData);
	else
		self:fLog("## MPMoney ## ERROR : Can not send client to client !");
	end;

end;


function MPMoney:SendToServer(actionSource, actionType, actionData)

	if g_currentMission:getIsServer() then
		self:ServerController(actionSource, actionType, actionData);
	else
		if not (type(actionSource)=="string")then
			actionSource = tostring(actionSource);
		end;
		if not (type(actionType)=="string")then
			actionType = tostring(actionType);
		end;
		if not (type(actionData)=="string")then
			actionData = tostring(actionData);
		end;
		ServerControllerEvent.sendEvent(actionSource, actionType, actionData);
	end

end;

function MPMoney:ClientController(actionTarget, actionType, actionData)
	
	if (actionTarget == self.username or actionTarget == "%all") then
		if actionType == "refreshBank" then
			self.bankMoney = actionData;
		elseif actionType == "refreshRanks" then
			self.ranks = actionData;
		elseif actionType == "refreshHistory" then
			self.history = "H#"..actionData;
		else
			self:fLog("## MPMoney ## ERROR : Invalid action '"..actionType.."' from server");
		end;
	end;
	
end;


function MPMoney:ServerController(actionSender, actionType, actionData)
	
	-- Check if database already exist
	self:checkDataFile();
	
	-- Get data file -- Get data file
    local xml = loadXMLFile("mpmoney_db", self.dbfilename);
	
	-- Update user list informations
		
	if actionType == "checkMe" then
		
		if not (actionSender == "") then

			local listPlayers = Utils.getNoNil(getXMLString(xml, "mpmoney_db.players_v8"), "");
			listPlayers = dec(listPlayers);

			local usernameExist = false;
			local isFirst = true;
			for player in string.gmatch(listPlayers, "([^#]+)") do
				isFirst = false;
				if (player:lower() == actionSender:lower()) then
					usernameExist = true;
					break;
				end;
			end;

			if not usernameExist then

				if not isFirst then
					listPlayers = listPlayers.."#";
				end;
				listPlayers = listPlayers..actionSender

				listPlayers = enc(listPlayers);

				setXMLString(xml, "mpmoney_db.players_v8", listPlayers);
				saveXMLFile(xml);

				self:fLog("## MPMoney ## Add new player '"..actionSender.."' in database");
				
				-- Get and send new ranks to all players
				self:SendToServer(self.username, "needRefreshRanks", "");

			end;

		end;
		
	elseif actionType == "needRefreshHistory" then
	
		-- Get history
		local clean_username = cleanup_string(tostring(actionSender));
		local history = Utils.getNoNil(getXMLString(xml, "mpmoney_db."..tostring(clean_username).."_history"), "");
		history = dec(history);
		self:SendToClient(actionSender, "refreshHistory", history);
		
	elseif actionType == "needRefreshBank" then
	
		-- Send player bank informations
		local senderBankMoney = self:getBankMoney(xml, actionSender);
		self:SendToClient(actionSender, "refreshBank", senderBankMoney);
		
	elseif actionType == "needRefreshRanks" then
	
			-- Get ranks
			local listPlayers = Utils.getNoNil(getXMLString(xml, "mpmoney_db.players_v8"), "");
			listPlayers = dec(listPlayers);
			local ranks = {};
			for player in string.gmatch(listPlayers, "([^#]+)") do
				ranks[player] = self:getBankMoney(xml, player);
			end;
			-- Sorted ranks
			local sortedRanks = "";
			local isFirst = true;
			for k,v in spairs(ranks, function(t,a,b) return t[b] < t[a] end) do
				if not isFirst then
					sortedRanks = sortedRanks.."#";
				else
					isFirst = false;
				end;
				sortedRanks = sortedRanks..k.."|"..tostring(v);
			end;
			-- Send ranks to all player
			self:SendToClient("%all", "refreshRanks", sortedRanks);
	
	elseif actionType == "sendMoney" then
	
		-- Découpe de actionData ICI
		local transfertType, targetUsername, moneyAmmount = actionData:match("([^,]+)|([^,]+)|([^,]+)");
		moneyAmmount = tonumber(moneyAmmount);
	
		-- Get source bank money
		local sourceBankMoney = self:getBankMoney(xml, actionSender);
		-- Get source server money
		local sourceWalletMoney = self:getWalletMoney(actionSender);
	
		if (actionSender == targetUsername) then

			-- To bank
			if ((transfertType == "toBank") and moneyAmmount > 0 and (moneyAmmount < (sourceWalletMoney+1))) then
				-- Remove money from source
				sourceWalletMoney=sourceWalletMoney-moneyAmmount;
				self:setWalletMoney(actionSender, sourceWalletMoney);
				-- Add money to target
				sourceBankMoney=sourceBankMoney+moneyAmmount;
				self:setBankMoney(xml, actionSender, sourceBankMoney);
				-- Refresh source bank informations
				self:SendToClient(actionSender, "refreshBank", sourceBankMoney);
				-- Log
				self:fLog("## MPMoney ## Transaction : "..actionSender.." transfer "..tostring(moneyAmmount).." euro from wallet to bank");
				-- Add information to history
				self:addInHistory(xml, transfertType, actionSender, targetUsername, moneyAmmount);
			end;

			-- To server
			if ((transfertType == "toServer") and moneyAmmount > 0 and (moneyAmmount < (sourceBankMoney+1))) then
				-- Remove money from source
				sourceBankMoney=sourceBankMoney-moneyAmmount;
				self:setBankMoney(xml, actionSender, sourceBankMoney);
				-- Add money to target informations
				sourceWalletMoney=sourceWalletMoney+moneyAmmount;
				self:setWalletMoney(actionSender, sourceWalletMoney);
				-- Refresh source bank informations
				self:SendToClient(actionSender, "refreshBank", sourceBankMoney);
				-- Log
				self:fLog("## MPMoney ## Transaction : "..actionSender.." transfer "..tostring(moneyAmmount).." euro from bank to wallet");
				-- Add information to history
				self:addInHistory(xml, transfertType, actionSender, targetUsername, moneyAmmount);
			end;

		else

			-- Get target bank money
			local targetBankMoney = self:getBankMoney(xml, targetUsername);
			-- Get target server money
			local targetWalletMoney = self:getWalletMoney(targetUsername);

			if (moneyAmmount > 0 and (moneyAmmount < (sourceBankMoney+1))) then
				-- Remove money from source
				sourceBankMoney=sourceBankMoney-moneyAmmount;
				self:setBankMoney(xml, actionSender, sourceBankMoney);
				-- Add money to target
				targetBankMoney=targetBankMoney+moneyAmmount;
				self:setBankMoney(xml, targetUsername, targetBankMoney);
				-- Refresh source bank
				self:SendToClient(actionSender, "refreshBank", sourceBankMoney);
				-- Refresh target bank
				self:SendToClient(targetUsername, "refreshBank", targetBankMoney);
				-- Log
				self:fLog("## MPMoney ## Transaction : "..actionSender.." transfer "..tostring(moneyAmmount).." euro from bank to "..targetUsername.." bank");
				-- Add information to history
				self:addInHistory(xml, transfertType, actionSender, targetUsername, moneyAmmount);
			end;

		end;
		
		-- Send new ranks to all players
		self:SendToServer(self.username, "needRefreshRanks", "");
	
    else
		self:fLog("## MPMoney ## ERROR : Invalid action '"..actionType.."' from "..actionSender);
    end;
	
	-- Close data file
    delete(xml);

	
end;


function MPMoney:getKeyString(sym)

	if sym == Input.KEY_0 then
		return ("0");
	end;
	if sym == Input.KEY_1 then
		return ("1");
	end;
	if sym == Input.KEY_2 then
		return ("2");
	end;
	if sym == Input.KEY_3 then
		return ("3");
	end;
	if sym == Input.KEY_4 then
		return ("4");
	end;
	if sym == Input.KEY_5 then
		return ("5");
	end;
	if sym == Input.KEY_6 then
		return ("6");
	end;
	if sym == Input.KEY_7 then
		return ("7");
	end;
	if sym == Input.KEY_8 then
		return ("8");
	end;
	if sym == Input.KEY_9 then
		return ("9");
	end;
	
	if sym == Input.KEY_KP_0 then
		return ("0");
	end;
	if sym == Input.KEY_KP_1 then
		return ("1");
	end;
	if sym == Input.KEY_KP_2 then
		return ("2");
	end;
	if sym == Input.KEY_KP_3 then
		return ("3");
	end;
	if sym == Input.KEY_KP_4 then
		return ("4");
	end;
	if sym == Input.KEY_KP_5 then
		return ("5");
	end;
	if sym == Input.KEY_KP_6 then
		return ("6");
	end;
	if sym == Input.KEY_KP_7 then
		return ("7");
	end;
	if sym == Input.KEY_KP_8 then
		return ("8");
	end;
	if sym == Input.KEY_KP_9 then
		return ("9");
	end;
	
	return "";
	
end;


function MPMoney:keyEvent(unicode, sym, modifier, isDown)

	if isDown then
		if self.HUDActive then
			if (self.inputSelected == 1) then
				if sym == Input.KEY_backspace then
					self.addToBank = string.sub (self.addToBank, 0, string.len(self.addToBank)-1);
					if (self.addToBank == "") then
						self.addToBank = 0;
					end;
				else
					local inputChar = self:getKeyString(sym);
					if not (inputChar == "") then
						self.addToBank = self.addToBank .. inputChar;
					end;
				end;
			end;
			if (self.inputSelected == 2) then
				if sym == Input.KEY_backspace then
					self.addToServer = string.sub (self.addToServer, 0, string.len(self.addToServer)-1);
					if (self.addToServer == "") then
						self.addToServer = 0;
					end;
				else
					local inputChar = self:getKeyString(sym);
					if not (inputChar == "") then
						self.addToServer = self.addToServer .. inputChar;
					end;
				end;
			end;
			if (self.inputSelected == 3) then
				if sym == Input.KEY_backspace then
					self.addToOther = string.sub (self.addToOther, 0, string.len(self.addToOther)-1);
					if (self.addToOther == "") then
						self.addToOther = 0;
					end;
				else
					local inputChar = self:getKeyString(sym);
					if not (inputChar == "") then
						self.addToOther = self.addToOther .. inputChar;
					end;
				end;
			end;
		end;
		if not (self.userChecked) then
			self:SendToServer(self.username, "checkMe","");
			self.userChecked = true;
		end;
	end;


end;


function MPMoney:deleteMap()
end;


addModEventListener(MPMoney);

